@@ -5,7 +5,7 @@
 
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
-# published by the Free Software Foundation; either version 2 of
+# published by the Free Software Foundation; either version 3 of
 # the License, or (at your option) any later version.
 #
 # This program is distributed in the hope that it will be
@@ -14,9 +14,7 @@
 # PURPOSE. See the GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-# MA 02110-1301, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import unittest
 import palpy as pal
@@ -69,6 +67,18 @@ class TestPAL(unittest.TestCase) :
         self.assertAlmostEqual( pal.airmas( 1.2354 ),
                                 3.015698990074724, 11 );
 
+    def test_altaz(self):
+        (az, azd, azdd, el, eld, eldd, pa, pad, padd) = pal.altaz( 0.7, -0.7, -0.65 )
+        self.assertAlmostEqual( az, 4.400560746660174, 12 )
+        self.assertAlmostEqual( azd, -0.2015438937145421, 12 )
+        self.assertAlmostEqual( azdd, -0.4381266949668748, 13 )
+        self.assertAlmostEqual( el, 1.026646506651396, 12 )
+        self.assertAlmostEqual( eld, -0.7576920683826450, 13 )
+        self.assertAlmostEqual( eldd, 0.04922465406857453, 14 )
+        self.assertAlmostEqual( pa, 1.707639969653937, 12 )
+        self.assertAlmostEqual( pad, 0.4717832355365627, 13 )
+        self.assertAlmostEqual( padd, -0.2957914128185515, 13 )
+
     def test_amp(self):
         (rm, dm) = pal.amp( 2.345, -1.234, 50100., 1990. )
         self.assertAlmostEqual( rm, 2.344472180027961, 6 )
@@ -83,6 +93,55 @@ class TestPAL(unittest.TestCase) :
         self.assertAlmostEqual( rm, 1.2335120411026936349, 11 )
         self.assertAlmostEqual( dm, -0.56702908706930343907, 11 )
 
+    def test_aopqkVector(self):
+        date = 51000.1
+        dut = 25.0
+        elongm = 2.1
+        phim = 0.5
+        hm = 3000.0
+        xp = -0.5e-6
+        yp = 1.0e-6
+        tdk = 280.0
+        pmb = 550.0
+        rh = 0.6
+        tlr = 0.006
+        wl=0.45
+        obsrms = pal.aoppa(date,dut,elongm,phim,hm,xp,yp,tdk,pmb,rh,wl,tlr)
+        np.random.seed(32)
+        nTests = 100
+        raIn = np.random.sample(nTests)*2.0*np.pi
+        decIn = (np.random.sample(nTests)-0.5)*np.pi
+        azControl = None
+        zeControl = None
+        haControl = None
+        dControl = None
+        rControl = None
+        for (rr,dd) in zip(raIn,decIn):
+            az,ze,ha,d,r = pal.aopqk(rr,dd,obsrms)
+            if azControl is None:
+                azControl = np.array([az])
+                zeControl = np.array([ze])
+                haControl = np.array([ha])
+                dControl = np.array([d])
+                rControl = np.array([r])
+            else:
+                azControl = np.append(azControl,az)
+                zeControl = np.append(zeControl,ze)
+                haControl = np.append(haControl,ha)
+                dControl = np.append(dControl,d)
+                rControl = np.append(rControl,r)
+
+        azTest,zeTest,haTest,dTest,rTest = pal.aopqkVector(raIn,decIn,obsrms)
+        for (a1,z1,h1,d1,r1,a2,z2,h2,d2,r2) in \
+            zip(azControl,zeControl,haControl,dControl,rControl,
+                azTest,zeTest,haTest,dTest,rTest):
+
+            self.assertAlmostEqual(a1,a2,12)
+            self.assertAlmostEqual(z1,z2,12)
+            self.assertAlmostEqual(h1,h2,12)
+            self.assertAlmostEqual(d1,d2,12)
+            self.assertAlmostEqual(r1,r2,12)
+
     def test_aop(self):
         dap = -0.1234
         date = 51000.1
@@ -301,6 +360,28 @@ class TestPAL(unittest.TestCase) :
         self.assertAlmostEqual( dh, hin, 12 )
         self.assertAlmostEqual( dd, din, 12 )
 
+    def test_de2hVector(self):
+        nTests = 100
+        phi = 0.35
+        np.random.seed(32)
+        raIn = np.random.random_sample(nTests)*np.pi*2.0
+        decIn = (np.random.random_sample(nTests)-0.5)*np.pi
+        azControl = None
+        elControl = None
+        for (rr,dd) in zip (raIn,decIn):
+            az, el = pal.de2h(rr, dd, phi)
+            if azControl is None:
+                azControl = np.array([az])
+                elControl = np.array([el])
+            else:
+                azControl = np.append(azControl,az)
+                elControl = np.append(elControl,el)
+
+        azTest, elTest = pal.de2hVector(raIn, decIn, phi)
+        for (a1,e1,a2,e2) in zip (azControl,elControl,azTest,elTest):
+            self.assertAlmostEqual(a1,a2,12)
+            self.assertAlmostEqual(e1,e2,12)
+
     def test_ecmat(self):
         expected = np.array( [
             [ 1.0,                    0.0,                   0.0 ],
@@ -507,6 +588,54 @@ class TestPAL(unittest.TestCase) :
         self.assertAlmostEqual( ra, 1.2344879748414849807, 7 )
         self.assertAlmostEqual( da, -0.56697099554368701746, 7 )
 
+    def test_mapqkzVector(self):
+        amprms = pal.mappa(2010, 55927)
+        np.random.seed(32)
+        nTests = 100
+        raIn = np.random.sample(nTests)*2.0*np.pi
+        decIn = (np.random.sample(nTests)-0.5)*np.pi
+        rControl=None
+        dControl=None
+        for (rr,dd) in zip(raIn,decIn):
+            r,d = pal.mapqkz(rr, dd, amprms)
+            if rControl is None:
+                rControl = np.array([r])
+                dControl = np.array([d])
+            else:
+                rControl = np.append(rControl,r)
+                dControl = np.append(dControl,d)
+
+        rTest,dTest = pal.mapqkzVector(raIn,decIn,amprms)
+        for (r1,d1,r2,d2) in zip(rControl,dControl,rTest,dTest):
+            self.assertAlmostEqual(r1,r2,12)
+            self.assertAlmostEqual(d1,d2,12)
+
+    def test_mapqkVector(self):
+        amprms = pal.mappa(2010, 55927)
+        np.random.seed(32)
+        nTests = 100
+        raIn = np.random.sample(nTests)*2.0*np.pi
+        decIn = (np.random.sample(nTests)-0.5)*np.pi
+        pmr = (np.random.sample(nTests)-0.5)*0.01
+        pmd = (np.random.sample(nTests)-0.5)*0.01
+        px = 0.00045+np.random.sample(nTests)*0.001
+        rv = 200.0*np.random.sample(nTests)
+        rControl=None
+        dControl=None
+        for (rr,dd,pr,pd,x,v) in zip(raIn,decIn,pmr,pmd,px,rv):
+            r,d = pal.mapqk(rr, dd, pr, pd, x, v, amprms)
+            if rControl is None:
+                rControl = np.array([r])
+                dControl = np.array([d])
+            else:
+                rControl = np.append(rControl,r)
+                dControl = np.append(dControl,d)
+
+        rTest,dTest = pal.mapqkVector(raIn,decIn,pmr,pmd,px,rv,amprms)
+        for (r1,d1,r2,d2) in zip(rControl,dControl,rTest,dTest):
+            self.assertAlmostEqual(r1,r2,12)
+            self.assertAlmostEqual(d1,d2,12)
+
     def test_moon(self):
         expected = np.array( [
              0.00229161514616454,
@@ -739,6 +868,33 @@ class TestPAL(unittest.TestCase) :
         self.assertAlmostEqual( ra, 0.01668919069414242368, 13 )
         self.assertAlmostEqual( dec, -1.093966454217127879, 13 )
 
+    def test_pmVector(self):
+        ep0 = 52000.0
+        ep1 = 53510.0
+        np.random.seed(32)
+        nTests = 100
+        raIn = np.random.sample(nTests)*2.0*np.pi
+        decIn = (np.random.sample(nTests)-0.5)*np.pi
+        pmr = 0.01*np.random.sample(nTests)
+        pmd = 0.01*np.random.sample(nTests)
+        px = 0.00045 + 0.001*np.random.sample(nTests)
+        rv = 1000.0*np.random.sample(nTests)
+        rControl = None
+        dControl = None
+        for (rr,dd,pr,pd,x,v) in zip(raIn,decIn,pmr,pmd,px,rv):
+            r, d = pal.pm(rr,dd,pr,pd,x,v,ep0,ep1)
+            if rControl is None:
+                rControl = np.array([r])
+                dControl = np.array([d])
+            else:
+                rControl = np.append(rControl,r)
+                dControl = np.append(dControl,d)
+
+        rTest, dTest = pal.pmVector(raIn,decIn,pmr,pmd,px,rv,ep0,ep1)
+        for (r1,d1,r2,d2) in zip(rControl,dControl,rTest,dTest):
+            self.assertAlmostEqual(r1,r2,12)
+            self.assertAlmostEqual(d1,d2,12)
+
     def test_prebn(self):
         expected = np.array( [
             [ 9.999257613786738e-1, -1.117444640880939e-2, -4.858341150654265e-3 ],
@@ -894,9 +1050,84 @@ class TestPAL(unittest.TestCase) :
         self.assertIsNone( dr02 )
         self.assertIsNone( dd02 )
 
-    def test_vecmat(self):
-        pass
+    def test_ds2tpVector(self):
+        raz = 0.3
+        decz = 0.1
+        nTests = 100
+        np.random.seed(32)
+        raIn = raz+(np.random.sample(nTests)-0.5)*0.2
+        decIn = decz+(np.random.sample(nTests)-0.5)*0.2
+        xiControl = None
+        etaControl = None
+        for (rr,dd) in zip(raIn,decIn):
+            xi, eta = pal.ds2tp(rr, dd, raz, decz)
+            if xiControl is None:
+                xiControl = np.array([xi])
+                etaControl = np.array([eta])
+            else:
+                xiControl = np.append(xiControl,xi)
+                etaControl = np.append(etaControl,eta)
 
+        xiTest, etaTest = pal.ds2tpVector(raIn, decIn, raz, decz)
+        for (x1,e1,x2,e2) in zip(xiControl,etaControl,xiTest,etaTest):
+            self.assertAlmostEqual(x1,x2,12)
+            self.assertAlmostEqual(e1,e2,12)
+
+    def test_vecmat(self):
+        # Not everything is implemented here
+        dav = np.array( [ -0.123, 0.0987, 0.0654 ] )
+        dav2m_expected = np.array( [
+            [  0.9930075842721269,  0.05902743090199868, -0.1022335560329612 ],
+            [ -0.07113807138648245, 0.9903204657727545,  -0.1191836812279541 ],
+            [  0.09420887631983825, 0.1256229973879967,   0.9875948309655174 ],
+        ] )
+
+        deuler_expected = np.array( [
+            [ -0.1681574770810878,  0.1981362273264315,  0.9656423242187410 ],
+            [ -0.2285369373983370,  0.9450659587140423, -0.2337117924378156 ],
+            [ -0.9589024617479674, -0.2599853247796050, -0.1136384607117296 ] ] )
+
+        dmxm_expected = np.array( [
+            [ -0.09010460088585805,  0.3075993402463796,  0.9472400998581048 ],
+            [ -0.3161868071070688,   0.8930686362478707, -0.3200848543149236 ],
+            [ -0.9444083141897035,  -0.3283459407855694,  0.01678926022795169 ] ] )
+
+        drm1 = pal.dav2m( dav )
+        np.testing.assert_array_almost_equal( drm1, dav2m_expected, decimal=12 )
+
+        drm2 = pal.deuler( "YZY", 2.345, -0.333, 2.222 )
+        np.testing.assert_array_almost_equal( drm2, deuler_expected, decimal=12 )
+
+        drm = pal.dmxm( drm2, drm1 )
+        np.testing.assert_array_almost_equal( drm, dmxm_expected, decimal=12 )
+
+        dv1 = pal.dcs2c( 3.0123, -0.999 )
+        dcs2c_expected = np.array( [-0.5366267667260525, 0.06977111097651444, -0.8409302618566215] )
+        np.testing.assert_array_almost_equal( dv1, dcs2c_expected, decimal=12 )
+
+        dv2 = pal.dmxv( drm1, dv1 )
+        dv3 = pal.dmxv( drm2, dv2 )
+        dmxv_expected = np.array( [-0.7267487768696160, 0.5011537352639822, 0.4697671220397141] )
+        np.testing.assert_array_almost_equal( dv3, dmxv_expected, decimal=12 )
+
+        dv4 = pal.dimxv( drm, dv3 )
+        dimxv_expected = np.array( [ -0.5366267667260526, 0.06977111097651445, -0.8409302618566215 ] )
+        np.testing.assert_array_almost_equal( dv4, dimxv_expected, decimal=12 )
+
+        dv5 = pal.dm2av( drm )
+        dm2av_expected = np.array( [ 0.006889040510209034, -1.577473205461961, 0.5201843672856759 ] )
+        np.testing.assert_array_almost_equal( dv5, dm2av_expected, decimal=12 )
+
+        dv5 *= 1000.0
+
+        (dv6, dvm) = pal.dvn( dv5 )
+        dvn_expected = np.array( [0.004147420704640065, -0.9496888606842218, 0.3131674740355448] )
+        np.testing.assert_array_almost_equal( dv6, dvn_expected, decimal=12 )
+        self.assertAlmostEqual( dvm, 1661.042127339937, 9 )
+
+        dv7 = pal.dvxv( dv6, dv1 )
+        dvxv_expected = np.array( [ 0.7767720597123304, -0.1645663574562769, -0.5093390925544726 ] )
+        np.testing.assert_array_almost_equal( dv7, dvxv_expected, decimal=12 )
 
 if __name__ == '__main__':
     unittest.main()
